﻿local NekoLoot = LibStub("AceAddon-3.0"):GetAddon("NekoLoot")
local L = LibStub("AceLocale-3.0"):GetLocale("NekoLoot")


------
-- constants
local FreeSlots = GetContainerNumFreeSlots

local lootMode = nil
local clearMode = nil
local errorDialog = nil
local lootMethod
local wasBind = false


------
-- Changes the modus LOOT_OPEN event is handled by this addon.
--
-- mode				: the mode which state should be changed
-- enable			: true if given mode should be enabled
function NekoLoot:ChangeLootOpen(mode, enable)
	NekoLoot:UnregisterEvent("LOOT_OPENED")
	local text = ""

	if mode == "fast" and enable then 
		mode, text = "FastLootOpen", "Fast"
		NekoLoot:RegisterEvent("LOOT_BIND_CONFIRM", "ConfirmBind")
	elseif mode == "fast" and not enable then
		mode, text = "NormalLootOpen", "Normal"
		NekoLoot:UnregisterEvent("LOOT_BIND_CONFIRM", "ConfirmBind")
	
	elseif mode == "extraFast" and enable then
		mode, text = "ExtraFastLootOpen", "ExtraFast"
		NekoLoot:RegisterEvent("LOOT_BIND_CONFIRM", "ConfirmBind")
		NekoLoot:RegisterEvent("UI_ERROR_MESSAGE", "LootOpenOnError")
	elseif mode == "extraFast" and not enable then
		mode, text = "NormalLootOpen", "Normal"
		NekoLoot:UnregisterEvent("LOOT_BIND_CONFIRM", "ConfirmBind")
		NekoLoot:UnregisterEvent("UI_ERROR_MESSAGE", "LootOpenOnError")
	
	else 
		mode, text = "NormalLootOpen", "Normal"
	end
	
	-- report change to user
	if mode ~= self.lootMode and not (self.lootMode == nil and mode == "NormalLootOpen") then
		NekoLoot:Print(L["LootingChanged"] .. text)
	end
	
	-- store mode to check if changed later
	self.lootMode = mode

	-- finaly register loot mode to event
	NekoLoot:RegisterEvent("LOOT_OPENED", mode)
end


------
-- Changes the modus LOOT_SLOT_CLEARED event is handled by this addon.
--
-- mode				: the mode which state should be changed
-- enable			: true if given mode should be enabled
function NekoLoot:ChangeLootSlotCleared(mode, enable)
	NekoLoot:UnregisterEvent("LOOT_SLOT_CLEARED")
	local text = ""

	if mode == "collapse" and enable then
		mode, text = "LootSlotClearedCollapse", L["Collapse"]
	elseif mode == "fastCollapse" and enable then
		mode, text = "FastLootSlotClearedCollapse", L["FastCollapse"]
	else
		mode, text = "LootSlotCleared", L["Normal"]
	end
	
	-- report change to user
	if mode ~= self.clearMode and not (self.clearMode == nil and mode == "LootSlotCleared") then
		NekoLoot:Print(L["ClearingChanged"] .. text)
	end
	
	-- store mode to check if changed later
	self.clearMode = mode

	-- finaly register loot mode to event
	NekoLoot:RegisterEvent("LOOT_SLOT_CLEARED", mode)
end


------
-- Stores current loot method
function NekoLoot:LootMethodChanged()
	self.lootMethod = GetLootMethod()
end


------
-- Stores if a LOOT_BIND_CONFIRM event was fired. Next loot window must be
-- shown to allow looting item.
function NekoLoot:ConfirmBind(event, index)
	self.wasBind = true
	NekoLoot:Print(L["OpenAgain"])
end


------
-- If "bind on pickup" item is in loot and item still known then disable autoloot
function NekoLoot:ProtectAgainstAutoloot()
	local numLoot = GetNumLootItems()
	local bindText, known = "", false
	
	for index = 1, numLoot do
		itemLink = GetLootSlotLink(index)
		bindText, _, _, _, known = NekoLoot:GetItemBinding(itemLink)
		if bindText == L["BoP"] and known then
			PlaySoundFile("Sound\\Spells\\PVPFlagTakenHorde.wav")
			SetCVar("AutoLootDefault", 0)
			break
		end
	end
end


local function ReEnableExtraFastLooting()
	local empty = 0
	if NekoLoot.db.profile.lootMode.wasExtraFast then
		for index = 1, NUM_BAG_SLOTS do
			empty = empty + FreeSlots(index)
		end
		if empty >= 2 then
			NekoLoot:ChangeLootOpen("extraFast", true)
			NekoLoot.db.profile.lootMode.extraFast = true
			NekoLoot.db.profile.lootMode.wasExtraFast = false
		end
	end

end

------
-- Simple show lootframe
function NekoLoot:NormalLootOpen()
	if NekoLoot.db.profile.lootMode.protectAgainstAuto then NekoLoot:ProtectAgainstAutoloot() end
	ReEnableExtraFastLooting()
	
	if IsFishingLoot() then
		PlaySound("FISHING REEL IN")
	end
	self:Update()
	self.wasBind = false
end


------
-- Check if lootframe is needed and if show it.
function NekoLoot:FastLootOpen()
	if NekoLoot.db.profile.lootMode.protectAgainstAuto then NekoLoot:ProtectAgainstAutoloot() end
	
	if IsFishingLoot() then
		PlaySound("FISHING REEL IN")
	end

	local empty, num = 0, GetNumLootItems()
	local modified = IsModifiedClick(AUTOLOOTTOGGLE)
	local default = GetCVar("autoLootDefault") == "1"

	-- autoloot
	-- try to hurry up looting
	if (modified or default) and not (modified and default) then
		for index = 1, NUM_BAG_SLOTS do
			empty = empty + FreeSlots(index)
		end
		if num <= empty and not (self.lootMethod == "master") and not self.wasBind then
			CloseLoot()
		else
			self:Update()
			self.wasBind = false
		end
	-- no autoloot
	elseif num > 0 or self.wasBind then
		self:Update()
		self.wasBind = false
	else
		CloseLoot()
	end
end


------
-- Close loot frame to speed up auto loot
function NekoLoot:ExtraFastLootOpen()
	if NekoLoot.db.profile.lootMode.protectAgainstAuto then NekoLoot:ProtectAgainstAutoloot() end

	-- lootMethod is one of 'freeforall', 'roundrobin', 'master', 'group', 'needbeforegreed'
	if not (self.lootMethod == "master") and not self.wasBind then
		CloseLoot()
	else
		self:Update()
		self.wasBind = false
	end
end


------
-- ButtonHandler of the click button of loot error dialog.
-- Hides the dialog and changes loot mode to a slower but cleaner way.
local function ErrorDialogButtonHandler()
	if NekoLoot.dialog:IsShown() then
		NekoLoot.dialog:Hide()
		NekoLoot:ChangeLootOpen("extraFast", false)
		NekoLoot.db.profile.lootMode.extraFast = false
		NekoLoot.db.profile.lootMode.wasExtraFast = true
	end
end

------
------
-- Check given event for inventory is full msg and if create error dialog
--
-- event			: event to handle
-- msg				: msg of event
function NekoLoot:LootOpenOnError(event, msg, ...)
	if msg == ERR_INV_FULL then
		UIErrorsFrame:Clear()
		-- NekoLoot:Print(RED_FONT_COLOR_CODE.."Inventar voll! Disable ExtraFastLooting or Cleanup your Bags..."..FONT_COLOR_CODE_CLOSE)
		NekoLoot:CreateDialog(
			"NekoLootErrorDialog",
			ERR_INV_FULL .. L["Error. Disable ExtraFastLooting"],
			"OK",
			"BOTTOM",
			ErrorDialogButtonHandler)
	end
end


------
-- Resets ItemCount and hides the item button with given index.
--
-- index			: index of button that should be hided
function NekoLoot:ClearButtonFrame(index)
	if index == nil then return nil end
	local button = self.buttonFrames[index].button
	
	SetItemButtonCount(button, 0)
	button:Hide()
	self.buttonFrames[index]:Hide()
end


------
-- Clear a single loot slot after item was picked up.
-- This only hides the button of cleared loot slot.
--
-- info
-- index			: index of item in loot table that was picked up
function NekoLoot:LootSlotCleared(info, index)
	NekoLoot:ClearButtonFrame(index)
end


------
-- Clear a single loot slot after item was picked up.
-- This updates the hole window. (SLOW)
--
-- info
-- index			: index of item in loot table that was picked up
function NekoLoot:LootSlotClearedCollapse(info, index)
	self.Update()
end


------
-- Clear a single loot slot after item was picked up. This hides the
-- button of cleared loot slot. Sort item list new and resizes the window.
--
-- info
-- index			: index of item in loot table that was picked up
function NekoLoot:FastLootSlotClearedCollapse(info, index)
	local prev, next
	
	-- find button
	for frame_index, frame in ipairs(self.items) do
		if frame.button.slot == index then
			index = frame_index
			break
		end
	end
	
	-- adjust elements
	prev = self.items[index - 1]
	next = self.items[index + 1]

	-- if no frame is shown for slot return
	if self.items[index] == nil then return end

	-- hide button
	self.items[index]:Hide()
	table.remove(self.items, index)
	
	if prev and next then
		next:SetPoint("TOPLEFT", prev, "BOTTOMLEFT", 0, -2)
	elseif next then
		next:SetPoint("TOPLEFT", self.lootFrame, "TOPLEFT", 10, -10)
	end
	
	-- set new height
	self:SetLootFrameHeight(#self.items)
end


------
-- Loot frame closed. Clean up...
function NekoLoot:LootClosed()
	for index = 1, #self.buttonFrames do
		self:ClearButtonFrame(index)
	end
	
	-- if config mode keep frame visible
	if self.unlock_dialog and self.unlock_dialog:IsVisible() then
		return 
	end
	CloseLoot()
	self.lootFrame:Hide()
end